using System;
using System.IO;
using System.Collections;
using System.Threading;
using System.Diagnostics;
using System.Reflection;

using Server.Network;

namespace Server
{
	/// <summary>
	/// Performs case-insensitive string operations.
	/// </summary>
	public class Insensitive
	{
		private static IComparer m_Comparer = System.Collections.CaseInsensitiveComparer.Default;

		public static IComparer Comparer
		{
			get 
			{
				return Insensitive.m_Comparer;
			}
		}

		public static int Compare(string a, string b)
		{
			return Insensitive.m_Comparer.Compare(a,b);
		}

		public static bool Contains(string a, string b)
		{
			if (((a==null) || (b==null)) || (a.Length < b.Length))
			{
				return false;
			}
			a = a.ToLower();
			b = b.ToLower();
			return !(a.IndexOf(b)<0);
		}

		public static bool EndsWith(string a, string b)
		{
			if (((a==null) || (b==null)) || (a.Length < b.Length))
			{
				return false;
			}
			return (Insensitive.m_Comparer.Compare(a.Substring((a.Length - b.Length)), b) == 0);
		}

		public static bool Equals(string a, string b)
		{
			if ((a==null) && (b==null))
			{
				return true;
			}
			if ((a==null) || ((b==null) || (a.Length != b.Length)))
			{
				return false;
			}
			return (Insensitive.m_Comparer.Compare(a,b)==0);
		}

		public static bool StartsWith(string a, string b)
		{
			if (((a==null) || (b==null)) || (a.Length < b.Length))
			{
				return false;
			}
			return (Insensitive.m_Comparer.Compare(a.Substring(0,b.Length), b) == 0);
		}
	}

	public class MultiTextWriter : TextWriter
	{
		private ArrayList m_Streams;
		public override System.Text.Encoding Encoding
		{
			get
			{
				return System.Text.Encoding.Default;
			}
		}

		public MultiTextWriter(params TextWriter[] streams)
		{
			this.m_Streams = new ArrayList(streams);
			if (this.m_Streams.Count < 0)
			{
				throw new ArgumentException("You must specify at least one stream.");
			}
		} 

		public void Add(TextWriter tw)
		{
			this.m_Streams.Add(tw);
 		} 

		public void Remove(TextWriter tw)
		{
			this.m_Streams.Remove(tw);
 		} 

		public override void Write(char ch)
		{
			for (int streamNum = 0; (streamNum < this.m_Streams.Count); streamNum++)
			{
				((TextWriter) this.m_Streams[streamNum]).Write(ch);
			}
		} 

		public override void WriteLine(string line)
		{
			for (int streamNum = 0; (streamNum < this.m_Streams.Count); streamNum++)
			{
				((TextWriter) this.m_Streams[streamNum]).WriteLine(line);
			}
		} 

		public override void WriteLine(string line, params object[] args)
		{
			/*base.*/WriteLine(string.Format(line, args));
		} 
	}

	/// <summary>
	/// The core server class.
	/// </summary>
	public class Core
	{
		public Core() : base()
		{
		}

		private static void CurrentDomain_ProcessExit(object sender, EventArgs e)
		{
			Core.HandleClosed();
		}

		private static void CurrentDomain_UnhandledException(object sender, UnhandledExceptionEventArgs e)
		{
			bool flag1;
			CrashedEventArgs args1;
			Console.WriteLine((e.IsTerminating ? "Error:" : "Warning:"));
			Console.WriteLine(e.ExceptionObject);
			if (e.IsTerminating)
			{
				Core.m_Crashed = true;
				flag1 = false;
				try
				{
					args1 = new CrashedEventArgs((e.ExceptionObject as Exception));
					EventSink.InvokeCrashed(args1);
					flag1 = args1.Close;
 
				}
				catch /*(Exception ignored)*/
				{
  
				}
				if (!flag1 && !Core.m_Service)
				{
					Console.WriteLine("This exception is fatal, press return to exit");
					Console.ReadLine();
 
				}
 
			}
 
		} 

		public static string FindDataFile(string path)
		{
				string text1;
			int num1;
			if (Core.m_DataDirectories.Count == 0)
			{
				throw new InvalidOperationException("Attempted to FindDataFile before DataDirectories list has been filled.");
 
			}
			text1 = null;
			for (num1 = 0; (num1 < Core.m_DataDirectories.Count); num1 = (num1 + 1))
			{
				text1 = Path.Combine(((string) Core.m_DataDirectories[num1]), path);
				if (File.Exists(text1))
				{
					goto L_004F;
 
				}
				text1 = null;
 
			}
 
			L_004F:
				return text1; 
		} 

		public static string FindDataFile(string format, params object[] args)
		{
				return Core.FindDataFile(string.Format(format, args)); 
		} 

		private static void HandleClosed()
		{
				if (Core.m_Closing)
		  {
			  return; 
		  }
			Core.m_Closing = true;
			Console.Write("Exiting...");
			if (!Core.m_Crashed)
			{
				EventSink.InvokeShutdown(new ShutdownEventArgs());
 
			}
			Core.timerThread.Join();
			Console.WriteLine("done");
 
		} 

		public static void Main(string[] args)
		{
			bool flag1;
			int num1;
			Server.Timer.TimerThread thread1;
			string text1;
			int num2;
			Exception exception1;
			TextWriter[] array1;
			AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(Core.CurrentDomain_UnhandledException);
			AppDomain.CurrentDomain.ProcessExit += new EventHandler(Core.CurrentDomain_ProcessExit);
			flag1 = false;
			num1 = 0;
			goto L_0075;
 
			L_0032:
				if (Insensitive.Equals(args[num1], "-debug"))
				{
					flag1 = true;
 
				}
				else
				{
						if (Insensitive.Equals(args[num1], "-service"))
				  {
					  Core.m_Service = true;
					  goto L_0071;
 
				  }
					if (Insensitive.Equals(args[num1], "-profile"))
					{
						Core.Profiling = true;
 
					}
 
				}
 
			L_0071:
				num1 = (num1 + 1);
 
			L_0075:
				if (num1 < args.Length)
				{
					goto L_0032;
 
				}
			try
			{
				if (Core.m_Service)
				{
					if (!Directory.Exists("Logs"))
					{
						Directory.CreateDirectory("Logs");
 
					}
					array1 = new TextWriter[2];
					array1[0] = Console.Out;
					array1[1] = new FileLogger("Logs/Console.log");
					Core.m_MultiConOut = new MultiTextWriter(array1);
					Console.SetOut(new MultiTextWriter(array1));
 
				}
				else
				{
						array1 = new TextWriter[1];
					array1[0] = Console.Out;
					Core.m_MultiConOut = new MultiTextWriter(array1);
					Console.SetOut(new MultiTextWriter(array1));
 
				}
 
			}
			catch /*(Exception ignored)*/
			{
  
			}
			Core.m_Thread = Thread.CurrentThread;
			Core.m_Process = Process.GetCurrentProcess();
			Core.m_Assembly = Assembly.GetEntryAssembly();
			if (Core.m_Thread != null)
			{
				Core.m_Thread.Name = "Core Thread";
 
			}
			if (Core.BaseDirectory.Length > 0)
			{
				Directory.SetCurrentDirectory(Core.BaseDirectory);
 
			}
			thread1 = new Server.Timer.TimerThread();
			Core.timerThread = new Thread(new ThreadStart(thread1.TimerMain));
			Core.timerThread.Name = "Timer Thread";
			while (!ScriptCompiler.Compile(flag1))
			{
				Console.WriteLine("Scripts: One or more scripts failed to compile or no script files were found.");
				Console.WriteLine(" - Press return to exit, or R to try again.");
				text1 = Console.ReadLine();
				if ((text1 == null) || (text1.ToLower() != "r"))
				{
					return; 
				}
 
			}
			Region.Load();
			Core.m_MessagePump = new MessagePump(new Listener(Listener.Port));
			Core.timerThread.Start();
			for (num2 = 0; (num2 < Map.AllMaps.Count); num2 = (num2 + 1))
			{
				((Map) Map.AllMaps[num2]).Tiles.Force();
 
			}
			NetState.Initialize();
			EventSink.InvokeServerStarted();
			try
			{
				while (!Core.m_Closing)
				{
					Thread.Sleep(1);
					Mobile.ProcessDeltaQueue();
					Item.ProcessDeltaQueue();
					Timer.Slice();
					Core.m_MessagePump.Slice();
					NetState.ProcessDisposedQueue();
					if (Core.Slice != null)
					{
						Core.Slice();
					}
 
				}
 
			}
			catch (Exception exception2)
			{
				exception1 = exception2;
				Core.CurrentDomain_UnhandledException(null, new UnhandledExceptionEventArgs(exception1, true));
 
			}
			if (Core.timerThread.IsAlive)
			{
				Core.timerThread.Abort();
 
			}
 
		}
		
		public static void VerifySerialization()
		{
				int num1;
			Core.m_ItemCount = 0;
			Core.m_MobileCount = 0;
			Core.VerifySerialization(Assembly.GetCallingAssembly());
			for (num1 = 0; (num1 < ScriptCompiler.Assemblies.Length); num1 = (num1 + 1))
			{
				Core.VerifySerialization(ScriptCompiler.Assemblies[num1]);
 
			}
 
		} 

		private static void VerifySerialization(Assembly a)
		{
				Type[] array1;
			Type type1;
			bool flag1;
			bool flag2;
			Type[] array2;
			int num1;
			if (a == null)
			{
				return; 
			}
			array2 = new Type[1];
			array2[0] = typeof(Serial);
			array1 = array2;
			array2 = a.GetTypes();
			for (num1 = 0; (num1 < array2.Length); num1 = (num1 + 1))
			{
				type1 = array2[num1];
				flag1 = type1.IsSubclassOf(typeof(Item));
				if (flag1 || type1.IsSubclassOf(typeof(Mobile)))
				{
					if (flag1)
					{
						Core.m_ItemCount = (Core.m_ItemCount + 1);
 
					}
					else
					{
							Core.m_MobileCount = (Core.m_MobileCount + 1);
 
					}
					flag2 = false;
					try
					{
						if (type1.GetConstructor(array1) == null)
						{
							if (!flag2)
							{
								Console.WriteLine("Warning: {0}", type1);
 
							}
							flag2 = true;
							Console.WriteLine(" - No serialization constructor");
 
						}
						if (type1.GetMethod("Serialize", BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance | BindingFlags.DeclaredOnly) == null)
						{
							if (!flag2)
							{
								Console.WriteLine("Warning: {0}", type1);
 
							}
							flag2 = true;
							Console.WriteLine(" - No Serialize() method");
 
						}
						if (type1.GetMethod("Deserialize", BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance | BindingFlags.DeclaredOnly) == null)
						{
							if (!flag2)
							{
								Console.WriteLine("Warning: {0}", type1);
 
							}
							flag2 = true;
							Console.WriteLine(" - No Deserialize() method");
 
						}
						if (flag2)
						{
							Console.WriteLine();
 
						}
 
					}
					catch /*(Exception ignored)*/
					{
  
					}
 				}
 			}
 		}		

		public static bool AOS { 
			get
			{
				return Core.m_AOS;
			}
			set
			{
				Core.m_AOS = value;
			}
		}

		public static Assembly Assembly 
		{ 
			get
			{
				return Core.m_Assembly;
			}
			set
			{
				Core.m_Assembly = value;
			}
		}

		public static string BaseDirectory 
		{ 
			get
			{
				if (Core.m_BaseDirectory == null)
				{
					try
					{
						Core.m_BaseDirectory = Core.ExePath;
						if (Core.m_BaseDirectory.Length > 0)
						{
							Core.m_BaseDirectory = Path.GetDirectoryName(Core.m_BaseDirectory);
 
						}
 
					}
					catch /*(Exception ignored)*/
					{
						Core.m_BaseDirectory = "";
 
					}
 
				}
				return Core.m_BaseDirectory;
			}
		}

		public static bool Closing 
		{ 
			get
			{
				return Core.m_Closing;
			}
			set
			{
				Core.m_Closing = value;
			}
		}

		public static ArrayList DataDirectories { 
			get
			{
				return Core.m_DataDirectories;
			}
		}

		public static string ExePath 
		{ 
			get
			{
				if (Core.m_ExePath == null)
				{
					Core.m_ExePath = Process.GetCurrentProcess().MainModule.FileName;
 
				}
				return Core.m_ExePath;
			}
		}

		public static int GlobalMaxUpdateRange 
		{ 
			get
			{
				return Core.m_GlobalMaxUpdateRange;
			}
			set
			{
				Core.m_GlobalMaxUpdateRange = value;
			}
		}

		public static MessagePump MessagePump 
		{ 
			get
			{
				return Core.m_MessagePump;
			}
			set
			{
				Core.m_MessagePump = value;
			}
		}

		public static MultiTextWriter MultiConsoleOut 
		{ 
			get
			{
				return Core.m_MultiConOut;
			}
		}

		public static Process Process 
		{ 
			get
			{
				return Core.m_Process;
			}
		}

		public static TimeSpan ProfileTime 
		{ 
			get
			{
				if (Core.m_ProfileStart > DateTime.MinValue)
				{
					return Core.m_ProfileTime + (DateTime.Now - Core.m_ProfileStart); 
				}
				return Core.m_ProfileTime;
			}
		}

		public static bool Profiling 
		{ 
			get
			{
				return Core.m_Profiling;
			}
			set
			{
				if (Core.m_Profiling == value)
				{
					return; 
				}
				Core.m_Profiling = value;
				if (Core.m_ProfileStart > DateTime.MinValue)
				{
					Core.m_ProfileTime = Core.m_ProfileTime + (DateTime.Now - Core.m_ProfileStart);
 
				}
				Core.m_ProfileStart = (Core.m_Profiling ? DateTime.Now : DateTime.MinValue);
			}
		}

		public static int ScriptItems 
		{ 
			get
			{
				return Core.m_ItemCount;
			}
		}

		public static int ScriptMobiles 
		{ 
			get
			{
				return Core.m_MobileCount;
			}
		}

		public static bool Service 
		{ 
			get
			{
				return Core.m_Service;
			}
		}

		public static Thread Thread 
		{ 
			get
			{
				return Core.m_Thread;
			}
		}

		private static bool m_AOS;
		private static Assembly m_Assembly;
		private static string m_BaseDirectory;
		private static bool m_Closing;
		private static bool m_Crashed;
		private static ArrayList m_DataDirectories = new ArrayList();
		private static string m_ExePath;
		private static int m_GlobalMaxUpdateRange = 24;
		private static int m_ItemCount;
		private static MessagePump m_MessagePump;
		private static int m_MobileCount;
		private static MultiTextWriter m_MultiConOut;
		private static Process m_Process;
		private static DateTime m_ProfileStart;
		private static TimeSpan m_ProfileTime;
		private static bool m_Profiling;
		private static bool m_Service;
		private static Thread m_Thread;
		public static Slice Slice;
		private static Thread timerThread;
	}
}
