using System;

using Server;
using Server.Network;

namespace Server.Targeting
{
	public class LandTarget : IPoint3D
	{
		public LandTarget(Point3D location, Map map) : base()
		{
			Tile tile1;
			this.m_Location = location;
			if (map != null)
			{
				this.m_Location.Z = map.GetAverageZ(this.m_Location.X, this.m_Location.Y);
				tile1 = map.Tiles.GetLandTile(this.m_Location.X, this.m_Location.Y);
				this.m_TileID = (tile1.ID & 16383);
			}
		} 

		public TileFlag Flags { 
			get
			{
				return TileData.LandTable[this.m_TileID].Flags; 

			}
		}
		public Point3D Location 
		{ 
			get
			{
				return this.m_Location;
			}
		}
		public string Name 
		{ 
			get
			{
				return TileData.LandTable[this.m_TileID].Name;
			}
		}
		public int TileID 
		{ 
			get
			{
				return this.m_TileID;
			}
		}
		public int X 
		{ 
			get
			{
				return this.m_Location.X;
			}
		}
		public int Y 
		{ 
			get
			{
				return this.m_Location.Y;
			}
		}
		public int Z 
		{ 
			get
			{
				return this.m_Location.Z;
			}
		}

		private Point3D m_Location;
		private int m_TileID;
	}
 
	public abstract class MultiTarget : Target
	{
		public MultiTarget(int multiID, Point3D offset) : base(multiID, offset, 10, 1, 0)
		{
		} 

		public MultiTarget(int multiID, Point3D offset, int range, bool allowGround, TargetFlags flags)
			: base(range, allowGround, flags)
		{
			this.m_MultiID = multiID;
			this.m_Offset = offset;
		}

		public override Packet GetPacket()
		{
				return new MultiTargetReq(this); 
		} 

		public int MultiID 
		{ 
			get
			{
				return this.m_MultiID;
			}
			set
			{
				this.m_MultiID = value;
			}
		}
		public Point3D Offset 
		{ 
			get
			{
				return this.m_Offset;
			}
			set
			{
				this.m_Offset = value;
			}
		}

		private int m_MultiID;
		private Point3D m_Offset;
	}
 
	public class StaticTarget : IPoint3D
	{
		public StaticTarget(Point3D location, int itemID) : base()
		{
			this.m_Location = location;
			this.m_ItemID = (itemID & 16383);
			this.m_Location.Z = (this.m_Location.Z + TileData.ItemTable[this.m_ItemID].CalcHeight);
		}

		public TileFlag Flags 
		{ 
			get
			{
				return TileData.ItemTable[this.m_ItemID].Flags; 
			}
		}
		public int ItemID 
		{ 
			get
			{
				return this.m_ItemID;
			}
		}
		public Point3D Location 
		{ 
			get
			{
				return this.m_Location;
			}
		}
		public string Name 
		{ 
			get
			{
				return TileData.ItemTable[this.m_ItemID].Name; 
			}
		}
		public int X 
		{ 
			get
			{
				return this.m_Location.X;
			}
		}
		public int Y 
		{ 
			get
			{
				return this.m_Location.Y;
			}
		}
		public int Z 
		{ 
			get
			{
				return this.m_Location.Z;
			}
		}

		private int m_ItemID;
		private Point3D m_Location;
	}

	public abstract class Target
	{
		public Target(int range, bool allowGround, TargetFlags flags) : base()
		{
			Target.m_NextTargetID = (Target.m_NextTargetID + 1);
			this.m_TargetID = (Target.m_NextTargetID + 1);
			this.m_Range = range;
			this.m_AllowGround = allowGround;
			this.m_Flags = flags;
			this.m_CheckLOS = true;
		}

		public void BeginTimeout(Mobile from, TimeSpan delay)
		{
				this.m_TimeoutTime = DateTime.op_Addition(DateTime.Now, delay);
			if (this.m_TimeoutTimer != null)
			{
				this.m_TimeoutTimer.Stop();
 
			}
			this.m_TimeoutTimer = new TimeoutTimer(this, from, delay);
			this.m_TimeoutTimer.Start();
 
		} 

		public static void Cancel(Mobile m)
		{
				NetState state1;
			Target target1;
			state1 = m.NetState;
			if (state1 != null)
			{
				state1.Send(CancelTarget.Instance);
 
			}
			target1 = m.Target;
			if (target1 != null)
			{
				target1.OnTargetCancel(m, 1);
 
			}
 
		} 

		public void Cancel(Mobile from, TargetCancelType type)
		{
				this.CancelTimeout();
			from.ClearTarget();
			this.OnTargetCancel(from, type);
			this.OnTargetFinish(from);
 
		} 

		public void CancelTimeout()
		{
				if (this.m_TimeoutTimer != null)
		  {
			  this.m_TimeoutTimer.Stop();
 
		  }
			this.m_TimeoutTimer = null;
 
		} 

		public virtual Packet GetPacket()
		{
				return new TargetReq(this); 
		} 

		public void Invoke(Mobile from, object targeted)
		{
				Point3D pointd1;
			Map map1;
			Item item1;
			object obj1;
			this.CancelTimeout();
			from.ClearTarget();
			if (from.Deleted)
			{
				this.OnTargetCancel(from, 1);
				this.OnTargetFinish(from);
				return; 
			}
			if ((targeted as LandTarget) != null)
			{
				pointd1 = ((LandTarget) targeted).Location;
				map1 = from.Map;
 
			}
			else
			{
					if ((targeted as StaticTarget) != null)
			  {
				  pointd1 = ((StaticTarget) targeted).Location;
				  map1 = from.Map;
				  goto L_0155;
 
			  }
				if ((targeted as Mobile) != null)
				{
					if (((Mobile) targeted).Deleted)
					{
						this.OnTargetDeleted(from, targeted);
						this.OnTargetFinish(from);
						return; 
					}
					if (!((Mobile) targeted).CanTarget)
					{
						this.OnTargetUntargetable(from, targeted);
						this.OnTargetFinish(from);
						return; 
					}
					pointd1 = ((Mobile) targeted).Location;
					map1 = ((Mobile) targeted).Map;
					goto L_0155;
 
				}
				if ((targeted as Item) != null)
				{
					item1 = ((Item) targeted);
					if (item1.Deleted)
					{
						this.OnTargetDeleted(from, targeted);
						this.OnTargetFinish(from);
						return; 
					}
					if (!item1.CanTarget)
					{
						this.OnTargetUntargetable(from, targeted);
						this.OnTargetFinish(from);
						return; 
					}
					obj1 = item1.RootParent;
					if ((!this.m_AllowNonlocal && ((obj1 as Mobile) != null)) && ((obj1 != from) && (from.AccessLevel == 0)))
					{
						this.OnNonlocalTarget(from, targeted);
						this.OnTargetFinish(from);
						return; 
					}
					pointd1 = item1.GetWorldLocation();
					map1 = item1.Map;
					goto L_0155;
 
				}
				this.OnTargetCancel(from, 1);
				this.OnTargetFinish(from);
				return; 
			}
 
			L_0155:
				if ((map1 == null) || ((map1 != from.Map) || ((this.m_Range != -1) && !from.InRange(pointd1, this.m_Range))))
				{
					this.OnTargetOutOfRange(from, targeted);
 
				}
				else
				{
						if (!from.CanSee(targeted))
				  {
					  this.OnCantSeeTarget(from, targeted);
 
				  }
				  else
				  {
						  if (this.m_CheckLOS && !from.InLOS(targeted))
					{
						this.OnTargetOutOfLOS(from, targeted);
						goto L_0255;
 
					}
					  if (((targeted as Item) != null) && ((Item) targeted).InSecureTrade)
					  {
						  this.OnTargetInSecureTrade(from, targeted);
						  goto L_0255;
 
					  }
					  if (((targeted as Item) != null) && !((Item) targeted).IsAccessibleTo(from))
					  {
						  this.OnTargetNotAccessible(from, targeted);
						  goto L_0255;
 
					  }
					  if (((targeted as Item) != null) && !((Item) targeted).CheckTarget(from, this, targeted))
					  {
						  this.OnTargetUntargetable(from, targeted);
						  goto L_0255;
 
					  }
					  if (((targeted as Mobile) != null) && !((Mobile) targeted).CheckTarget(from, this, targeted))
					  {
						  this.OnTargetUntargetable(from, targeted);
						  goto L_0255;
 
					  }
					  if (from.Region.OnTarget(from, this, targeted))
					  {
						  this.OnTarget(from, targeted);
 
					  }
 
				  }
 
				}
 
			L_0255:
				this.OnTargetFinish(from);
 
		}
		
		protected virtual void OnCantSeeTarget(Mobile from, object targeted)
		{
				from.SendLocalizedMessage(500237);
 
		} 

		protected virtual void OnNonlocalTarget(Mobile from, object targeted)
		{
				from.SendLocalizedMessage(500447);
 
		} 

		protected virtual void OnTarget(Mobile from, object targeted)
		{  
		} 

		protected virtual void OnTargetCancel(Mobile from, TargetCancelType cancelType)
		{  
		} 

		protected virtual void OnTargetDeleted(Mobile from, object targeted)
		{  
		} 

		protected virtual void OnTargetFinish(Mobile from)
		{  
		} 

		protected virtual void OnTargetInSecureTrade(Mobile from, object targeted)
		{
				from.SendLocalizedMessage(500447);
 
		} 

		protected virtual void OnTargetNotAccessible(Mobile from, object targeted)
		{
				from.SendLocalizedMessage(500447);
 
		} 

		protected virtual void OnTargetOutOfLOS(Mobile from, object targeted)
		{
				from.SendLocalizedMessage(500237);
 
		} 

		protected virtual void OnTargetOutOfRange(Mobile from, object targeted)
		{
				from.SendLocalizedMessage(500446);
 
		} 

		protected virtual void OnTargetUntargetable(Mobile from, object targeted)
		{
				from.SendLocalizedMessage(500447);
 
		} 

		public void Timeout(Mobile from)
		{
				this.CancelTimeout();
			from.ClearTarget();
			Target.Cancel(from);
			this.OnTargetCancel(from, 3);
			this.OnTargetFinish(from);
 
		} 

		public bool AllowGround 
		{ 
			get
			{
				return this.m_AllowGround;
			}
			set
			{
				this.m_AllowGround = value;
			}
		}
		public bool AllowNonlocal 
		{ 
			get
			{
				return this.m_AllowNonlocal;
			}
			set
			{
				this.m_AllowNonlocal = value;
			}
		}
		public bool CheckLOS 
		{ 
			get
			{
				return this.m_CheckLOS;
			}
			set
			{
				this.m_CheckLOS = value;
			}
		}
		public bool DisallowMultis 
		{ 
			get
			{
				return this.m_DisallowMultis;
			}
			set
			{
				this.m_DisallowMultis = value;
			}
		}
		public TargetFlags Flags 
		{ 
			get
			{
				return this.m_Flags;
			}
			set
			{
				this.m_Flags = value;
			}
		}
		public int Range 
		{ 
			get
			{
				return this.m_Range;
			}
			set
			{
				this.m_Range = value;
			}
		}
		public int TargetID 
		{ 
			get
			{
				return this.m_TargetID();
			}
		}
		public DateTime TimeoutTime 
		{ 
			get
			{
				return this.m_TimeoutTime();
			}
		}

		private bool m_AllowGround;
		private bool m_AllowNonlocal;
		private bool m_CheckLOS;
		private bool m_DisallowMultis;
		private TargetFlags m_Flags;
		private static int m_NextTargetID;
		private int m_Range;
		private int m_TargetID;
		private DateTime m_TimeoutTime;
		private Timer m_TimeoutTimer;

		private class TimeoutTimer : Timer
		{
			public TimeoutTimer(Target target, Mobile m, TimeSpan delay) : base(delay)
			{
				this.m_Target = target;
				this.m_Mobile = m;
				if (TimeSpan.op_GreaterThanOrEqual(delay, TimeoutTimer.ThirtySeconds))
				{
					base.Priority = 6;
					return; 
				}
				if (TimeSpan.op_GreaterThanOrEqual(delay, TimeoutTimer.TenSeconds))
				{
					base.Priority = 5;
					return; 
				}
				if (TimeSpan.op_GreaterThanOrEqual(delay, TimeoutTimer.OneSecond))
				{
					base.Priority = 4;
					return; 
				}
				base.Priority = 2;
			} 

			protected override void OnTick()
			{
					if (this.m_Mobile.Target == this.m_Target)
			  {
				  this.m_Target.Timeout(this.m_Mobile);
 
			  }
 
			} 

			private Mobile m_Mobile;
			private Target m_Target;
			private static TimeSpan OneSecond = TimeSpan.FromSeconds(30);
			private static TimeSpan TenSeconds = TimeSpan.FromSeconds(10);
			private static TimeSpan ThirtySeconds = TimeSpan.FromSeconds(1);
		}
	}
 
	public enum TargetCancelType
	{
		Overriden = 0,
		Canceled = 1,
		Disconnected = 2,
		Timeout = 3,
	}
 
	public enum TargetFlags
	{
		None = 0,
		Harmful = 1,
		Beneficial = 2,
	}
}